@@ -1,4 +1,4 @@
-# Copyright (c) 2019 - 2023, Ilan Schnell; All Rights Reserved
+# Copyright (c) 2019 - 2025, Ilan Schnell; All Rights Reserved
 # bitarray is published under the PSF license.
 #
 # Author: Ilan Schnell
@@ -9,13 +9,14 @@ from __future__ import absolute_import
 
 import os
 import sys
+import random
 
 from bitarray import bitarray, bits2bytes
 
 from bitarray._util import (
-    zeros, ones, count_n, parity,
+    zeros, ones, count_n, parity, xor_indices,
     count_and, count_or, count_xor, any_and, subset,
-    _correspond_all,
+    correspond_all, byteswap,
     serialize, deserialize,
     ba2hex, hex2ba,
     ba2base, base2ba,
@@ -25,10 +26,11 @@ from bitarray._util import (
 )
 
 __all__ = [
-    'zeros', 'ones', 'urandom',
-    'pprint', 'make_endian', 'rindex', 'strip', 'count_n',
-    'parity', 'count_and', 'count_or', 'count_xor', 'any_and', 'subset',
-    'intervals',
+    'zeros', 'ones', 'urandom', 'random_p',
+    'pprint', 'strip', 'count_n',
+    'parity', 'xor_indices',
+    'count_and', 'count_or', 'count_xor', 'any_and', 'subset',
+    'correspond_all', 'byteswap', 'intervals',
     'ba2hex', 'hex2ba',
     'ba2base', 'base2ba',
     'ba2int', 'int2ba',
@@ -39,38 +41,137 @@ __all__ = [
 ]
 
 
-_is_py2 = bool(sys.version_info[0] == 2)
-
-
 def urandom(__length, endian=None):
-    """urandom(length, /, endian=None) -> bitarray
+    """urandom(n, /, endian=None) -> bitarray
 
-Return a bitarray of `length` random bits (uses `os.urandom`).
+Return random bitarray of length `n` (uses `os.urandom()`).
 """
-    a = bitarray(0, endian)
-    a.frombytes(os.urandom(bits2bytes(__length)))
+    a = bitarray(os.urandom(bits2bytes(__length)), endian)
     del a[__length:]
     return a
 
 
-def rindex(__a, __sub_bitarray=1, __start=0, __stop=sys.maxsize):
-    """rindex(bitarray, sub_bitarray=1, start=0, stop=<end>, /) -> int
+def random_p(__n, p=0.5, endian=None):
+    """random_p(n, /, p=0.5, endian=None) -> bitarray
 
-Return rightmost (highest) index where sub_bitarray (or item - defaults
-to 1) is found in bitarray (`a`), such that sub_bitarray is contained
-within `a[start:stop]`.
-Raises `ValueError` when the sub_bitarray is not present.
+Return pseudo-random bitarray of length `n`.  Each bit has probability `p` of
+being 1.  Equivalent to `bitarray((random() < p for _ in range(n)), endian)`.
+The bitarrays are reproducible when calling Python's `random.seed()` with a
+specific seed value.
+
+This function requires Python 3.12 or higher, as it depends on the standard
+library function `random.binomialvariate()`.  Raises `NotImplementedError`
+when Python version is too low.
 """
-    from warnings import warn
+    if sys.version_info[:2] < (3, 12):
+        raise NotImplementedError("bitarray.util.random_p() requires "
+                                  "Python 3.12 or higher")
+    r = _RandomP(__n, endian)
+    return r.random_p(p)
 
-    warn("rindex() is deprecated and will be removed in bitarray 3.0 - "
-         "use .index(..., right=True) method instead.",
-         DeprecationWarning, stacklevel=1)
+class _RandomP:
 
-    if not isinstance(__a, bitarray):
-        raise TypeError("bitarray expected, got '%s'" % type(__a).__name__)
+    # maximal number of calls to .random_half() in .combine()
+    M = 8
 
-    return __a.index(__sub_bitarray, __start, __stop, right=True)
+    # number of resulting probability intervals
+    K = 1 << M
+
+    # limit for setting individual bits randomly
+    SMALL_P = 0.01
+
+    def __init__(self, n=0, endian=None):
+        self.n = n
+        self.nbytes = bits2bytes(n)
+        self.endian = endian
+
+    def random_half(self):
+        """
+        Return bitarray with each bit having probability p = 1/2 of being 1.
+        """
+        # use randbytes() for reproducibility (not urandom())
+        a = bitarray(random.randbytes(self.nbytes), self.endian)
+        del a[self.n:]
+        return a
+
+    def get_op_seq(self, i):
+        """
+        Return bitarray containing operator sequence.
+        Each item represents a bitwise operation:   0: and   1: or
+        After applying the sequence (see .random_combine()), we
+        obtain a bitarray with probability  q = i / K
+        """
+        assert 0 < i < self.K
+        # sequence of &, | operations - least significant operations first
+        seq = int2ba(i, length=self.M, endian="little")
+        seq = strip(seq, mode="left")
+        del seq[0]
+        return seq
+
+    def random_combine(self, p):
+        """
+        Given a desired probability p, return a random bitarray with
+        actual probability  q = int(p * K) / K
+        """
+        a = self.random_half()
+        for k in self.get_op_seq(int(p * self.K)):
+            b = self.random_half()
+            if k:
+                a |= b
+            else:
+                a &= b
+        return a
+
+    def random_m(self, m):
+        """
+        Return a bitarray (of length n) with m bits randomly set to 1.
+        Designed for small m (compared to n).
+        """
+        assert 0 <= m <= self.n
+        a = zeros(self.n, self.endian)
+        for _ in range(m):
+            while 1:
+                i = random.randrange(self.n)
+                if not a[i]:
+                    a[i] = 1
+                    break
+        return a
+
+    def random_p(self, p):
+        # error check inputs and handle edge cases
+        if p <= 0.0 or p == 0.5 or p >= 1.0:
+            if p == 0.0:
+                return zeros(self.n, self.endian)
+            if p == 0.5:
+                return self.random_half()
+            if p == 1.0:
+                return ones(self.n, self.endian)
+            raise ValueError("p must be in range 0.0 <= p <= 1.0, got %s" % p)
+
+        # for small n, use literal definition
+        if self.n < 10:
+            return bitarray((random.random() < p for _ in range(self.n)),
+                            self.endian)
+
+        # exploit symmetry to establish: p <= 0.5
+        if p > 0.5:
+            a = self.random_p(1.0 - p)
+            a.invert()
+            return a
+
+        # for small p, set randomly individual bits
+        if p < self.SMALL_P:
+            return self.random_m(random.binomialvariate(self.n, p))
+
+        # combine random bitarrays using bitwise & and | operations
+        a = self.random_combine(p)
+        q = int(p * self.K) / self.K
+        if q < p:
+            # increase probability q by "oring" with probability x
+            x = (p - q) / (1.0 - q)
+            a |= self.random_p(x)
+
+        return a
 
 
 def pprint(__a, stream=None, group=8, indent=4, width=80):
@@ -129,30 +230,6 @@ function `pprint.pprint()`.
     stream.flush()
 
 
-def make_endian(__a, endian):
-    """make_endian(bitarray, /, endian) -> bitarray
-
-When the endianness of the given bitarray is different from `endian`,
-return a new bitarray, with endianness `endian` and the same elements
-as the original bitarray.
-Otherwise (endianness is already `endian`) the original bitarray is returned
-unchanged.
-"""
-    from warnings import warn
-
-    warn("make_endian() is deprecated and will be removed in bitarray 3.0 - "
-         "use bitarray(..., endian=...) instead",
-         DeprecationWarning, stacklevel=1)
-
-    if not isinstance(__a, bitarray):
-        raise TypeError("bitarray expected, got '%s'" % type(__a).__name__)
-
-    if __a.endian() == endian:
-        return __a
-
-    return bitarray(__a, endian)
-
-
 def strip(__a, mode='right'):
     """strip(bitarray, /, mode='right') -> bitarray
 
@@ -160,7 +237,8 @@ Return a new bitarray with zeros stripped from left, right or both ends.
 Allowed values for mode are the strings: `left`, `right`, `both`
 """
     if not isinstance(mode, str):
-        raise TypeError("str expected for mode, got '%s'" % type(__a).__name__)
+        raise TypeError("str expected for mode, got '%s'" %
+                        type(__a).__name__)
     if mode not in ('left', 'right', 'both'):
         raise ValueError("mode must be 'left', 'right' or 'both', got %r" %
                          mode)
@@ -210,20 +288,13 @@ The bit-endianness of the bitarray is respected.
     if length == 0:
         raise ValueError("non-empty bitarray expected")
 
-    le = bool(__a.endian() == 'little')
     if __a.padbits:
-        pad = zeros(__a.padbits, __a.endian())
-        __a = __a + pad if le else pad + __a
+        pad = zeros(__a.padbits, __a.endian)
+        __a = __a + pad if __a.endian == "little" else pad + __a
 
-    if _is_py2:
-        a = bitarray(__a, 'big')
-        if le:
-            a.reverse()
-        res = int(ba2hex(a), 16)
-    else: # py3
-        res = int.from_bytes(__a.tobytes(), byteorder=__a.endian())
+    res = int.from_bytes(__a.tobytes(), byteorder=__a.endian)
 
-    if signed and res >= 1 << (length - 1):
+    if signed and res >> length - 1:
         res -= 1 << length
     return res
 
@@ -231,63 +302,49 @@ The bit-endianness of the bitarray is respected.
 def int2ba(__i, length=None, endian=None, signed=False):
     """int2ba(int, /, length=None, endian=None, signed=False) -> bitarray
 
-Convert the given integer to a bitarray (with given endianness,
+Convert the given integer to a bitarray (with given bit-endianness,
 and no leading (big-endian) / trailing (little-endian) zeros), unless
 the `length` of the bitarray is provided.  An `OverflowError` is raised
 if the integer is not representable with the given number of bits.
 `signed` determines whether two's complement is used to represent the integer,
 and requires `length` to be provided.
 """
-    if not isinstance(__i, (int, long) if _is_py2 else int):
+    if not isinstance(__i, int):
         raise TypeError("int expected, got '%s'" % type(__i).__name__)
     if length is not None:
         if not isinstance(length, int):
-            raise TypeError("int expected for length")
+            raise TypeError("int expected for argument 'length'")
         if length <= 0:
             raise ValueError("length must be > 0")
-    if signed and length is None:
-        raise TypeError("signed requires length")
-
-    if __i == 0:
-        # there are special cases for 0 which we'd rather not deal with below
-        return zeros(length or 1, endian)
 
     if signed:
-        m = 1 << (length - 1)
+        if length is None:
+            raise TypeError("signed requires argument 'length'")
+        m = 1 << length - 1
         if not (-m <= __i < m):
             raise OverflowError("signed integer not in range(%d, %d), "
                                 "got %d" % (-m, m, __i))
         if __i < 0:
             __i += 1 << length
     else:  # unsigned
-        if __i < 0:
-            raise OverflowError("unsigned integer not positive, got %d" % __i)
-        if length and __i >= (1 << length):
+        if length and __i >> length:
             raise OverflowError("unsigned integer not in range(0, %d), "
                                 "got %d" % (1 << length, __i))
 
     a = bitarray(0, endian)
-    le = bool(a.endian() == 'little')
-    if _is_py2:
-        s = hex(__i)[2:].rstrip('L')
-        a.extend(hex2ba(s, 'big'))
-        if le:
-            a.reverse()
-    else: # py3
-        b = __i.to_bytes(bits2bytes(__i.bit_length()), byteorder=a.endian())
-        a.frombytes(b)
-
+    b = __i.to_bytes(bits2bytes(__i.bit_length()), byteorder=a.endian)
+    a.frombytes(b)
+    le = a.endian == 'little'
     if length is None:
-        return strip(a, 'right' if le else 'left')
+        return strip(a, 'right' if le else 'left') if a else a + '0'
 
-    la = len(a)
-    if la > length:
-        a = a[:length] if le else a[-length:]
-    if la < length:
-        pad = zeros(length - la, a.endian())
-        a = a + pad if le else pad + a
-    assert len(a) == length
-    return a
+    if len(a) > length:
+        return a[:length] if le else a[-length:]
+    if len(a) == length:
+        return a
+    # len(a) < length, we need padding
+    pad = zeros(length - len(a), a.endian)
+    return a + pad if le else pad + a
 
 # ------------------------------ Huffman coding -----------------------------
 
@@ -301,9 +358,9 @@ and return its root node.
 
     class Node(object):
         """
-        A Node instance will either have a 'symbol' (leaf node) or
-        a 'child' (a tuple with both children) attribute.
-        The 'freq' attribute will always be present.
+        There are to tyes of Node instances (both have 'freq' attribute):
+          * leaf node: has 'symbol' attribute
+          * parent node: has 'child' attribute (tuple with both children)
         """
         def __lt__(self, other):
             # heapq needs to be able to compare the nodes
@@ -320,7 +377,7 @@ and return its root node.
     # repeat the process until only one node remains
     while len(minheap) > 1:
         # take the two nodes with lowest frequencies from the queue
-        # to construct a new node and push it onto the queue
+        # to construct a new parent node and push it onto the queue
         parent = Node()
         parent.child = heappop(minheap), heappop(minheap)
         parent.freq = parent.child[0].freq + parent.child[1].freq
@@ -335,15 +392,12 @@ def huffman_code(__freq_map, endian=None):
 
 Given a frequency map, a dictionary mapping symbols to their frequency,
 calculate the Huffman code, i.e. a dict mapping those symbols to
-bitarrays (with given endianness).  Note that the symbols are not limited
-to being strings.  Symbols may may be any hashable object (such as `None`).
+bitarrays (with given bit-endianness).  Note that the symbols are not limited
+to being strings.  Symbols may be any hashable object.
 """
     if not isinstance(__freq_map, dict):
         raise TypeError("dict expected, got '%s'" % type(__freq_map).__name__)
 
-    b0 = bitarray('0', endian)
-    b1 = bitarray('1', endian)
-
     if len(__freq_map) < 2:
         if len(__freq_map) == 0:
             raise ValueError("cannot create Huffman code with no symbols")
@@ -353,16 +407,17 @@ to being strings.  Symbols may may be any hashable object (such as `None`).
         # So we represent the symbol by a single code of length one, in
         # particular one 0 bit.  This is an incomplete code, since if a 1 bit
         # is received, it has no meaning and will result in an error.
-        return {list(__freq_map)[0]: b0}
+        sym = list(__freq_map)[0]
+        return {sym: bitarray('0', endian)}
 
     result = {}
 
     def traverse(nd, prefix=bitarray(0, endian)):
         try:                    # leaf
             result[nd.symbol] = prefix
-        except AttributeError:  # parent, so traverse each of the children
-            traverse(nd.child[0], prefix + b0)
-            traverse(nd.child[1], prefix + b1)
+        except AttributeError:  # parent, so traverse each child
+            traverse(nd.child[0], prefix + '0')
+            traverse(nd.child[1], prefix + '1')
 
     traverse(_huffman_tree(__freq_map))
     return result
@@ -397,18 +452,18 @@ Note: the two lists may be used as input for `canonical_decode()`.
         # now just simply record the length for reaching each symbol
         try:                    # leaf
             code_length[nd.symbol] = length
-        except AttributeError:  # parent, so traverse each of the children
+        except AttributeError:  # parent, so traverse each child
             traverse(nd.child[0], length + 1)
             traverse(nd.child[1], length + 1)
 
     traverse(_huffman_tree(__freq_map))
 
-    # we now have a mapping of symbols to their code length,
-    # which is all we need
+    # We now have a mapping of symbols to their code length, which is all we
+    # need to construct a list of tuples (symbol, code length) sorted by
+    # code length:
+    table = sorted(code_length.items(), key=lambda item: item[1])
 
-    table = sorted(code_length.items(), key=lambda item: (item[1], item[0]))
-
-    maxbits = max(item[1] for item in table)
+    maxbits = table[-1][1]
     codedict = {}
     count = (maxbits + 1) * [0]
 
